@@ -1,17 +1,20 @@
 package k8shandler
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"time"
 
-	"github.com/operator-framework/operator-sdk/pkg/sdk"
 	"github.com/sirupsen/logrus"
 	"k8s.io/apimachinery/pkg/api/errors"
+	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/util/retry"
+	"sigs.k8s.io/controller-runtime/pkg/client"
 
-	api "github.com/openshift/elasticsearch-operator/pkg/apis/elasticsearch/v1"
+	api "github.com/openshift/elasticsearch-operator/pkg/apis/logging/v1"
+	"github.com/openshift/elasticsearch-operator/pkg/logger"
 	apps "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -29,9 +32,11 @@ type deploymentNode struct {
 	currentRevision string
 
 	clusterSize int32
+
+	client client.Client
 }
 
-func (deploymentNode *deploymentNode) populateReference(nodeName string, node api.ElasticsearchNode, cluster *api.Elasticsearch, roleMap map[api.ElasticsearchNodeRole]bool, replicas int32) {
+func (deploymentNode *deploymentNode) populateReference(nodeName string, node api.ElasticsearchNode, cluster *api.Elasticsearch, roleMap map[api.ElasticsearchNodeRole]bool, replicas int32, client client.Client) {
 
 	labels := newLabels(cluster.Name, nodeName, roleMap)
 
@@ -59,13 +64,15 @@ func (deploymentNode *deploymentNode) populateReference(nodeName string, node ap
 		},
 		ProgressDeadlineSeconds: &progressDeadlineSeconds,
 		Paused:                  false,
-		Template:                newPodTemplateSpec(nodeName, cluster.Name, cluster.Namespace, node, cluster.Spec.Spec, labels, roleMap),
+		Template:                newPodTemplateSpec(nodeName, cluster.Name, cluster.Namespace, node, cluster.Spec.Spec, labels, roleMap, client),
 	}
 
 	addOwnerRefToObject(&deployment, getOwnerRef(cluster))
 
 	deploymentNode.self = deployment
 	deploymentNode.clusterName = cluster.Name
+
+	deploymentNode.client = client
 }
 
 func (current *deploymentNode) updateReference(node NodeTypeInterface) {
@@ -93,7 +100,7 @@ func (node *deploymentNode) state() api.ElasticsearchNodeStatus {
 	}*/
 
 	// check if the secretHash changed
-	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace)
+	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace, node.client)
 	if newSecretHash != node.secretHash {
 		rolloutForReload = v1.ConditionTrue
 	}
@@ -108,13 +115,13 @@ func (node *deploymentNode) state() api.ElasticsearchNodeStatus {
 }
 
 func (node *deploymentNode) delete() {
-	sdk.Delete(&node.self)
+	node.client.Delete(context.TODO(), &node.self)
 }
 
 func (node *deploymentNode) create() error {
 
 	if node.self.ObjectMeta.ResourceVersion == "" {
-		err := sdk.Create(&node.self)
+		err := node.client.Create(context.TODO(), &node.self)
 		if err != nil {
 			if !errors.IsAlreadyExists(err) {
 				return fmt.Errorf("Could not create node resource: %v", err)
@@ -130,8 +137,8 @@ func (node *deploymentNode) create() error {
 		}
 
 		// update the hashmaps
-		node.configmapHash = getConfigmapDataHash(node.clusterName, node.self.Namespace)
-		node.secretHash = getSecretDataHash(node.clusterName, node.self.Namespace)
+		node.configmapHash = getConfigmapDataHash(node.clusterName, node.self.Namespace, node.client)
+		node.secretHash = getSecretDataHash(node.clusterName, node.self.Namespace, node.client)
 	}
 
 	return node.pause()
@@ -139,7 +146,7 @@ func (node *deploymentNode) create() error {
 
 func (node *deploymentNode) waitForInitialRollout() error {
 	err := wait.Poll(time.Second*1, time.Second*30, func() (done bool, err error) {
-		if getErr := sdk.Get(&node.self); getErr != nil {
+		if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self); getErr != nil {
 			logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, getErr)
 			return false, getErr
 		}
@@ -166,7 +173,7 @@ func (node *deploymentNode) nodeRevision() string {
 
 func (node *deploymentNode) waitForNodeRollout(currentRevision string) error {
 	err := wait.Poll(time.Second*1, time.Second*30, func() (done bool, err error) {
-		if getErr := sdk.Get(&node.self); getErr != nil {
+		if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self); getErr != nil {
 			logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, getErr)
 			return false, getErr
 		}
@@ -195,7 +202,7 @@ func (node *deploymentNode) setPaused(paused bool) error {
 	nretries := -1
 	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
 		nretries++
-		if getErr := sdk.Get(pauseNode); getErr != nil {
+		if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: pauseNode.Name, Namespace: pauseNode.Namespace}, pauseNode); getErr != nil {
 			logrus.Debugf("Could not get Elasticsearch node resource %v: %v", pauseNode.Name, getErr)
 			return getErr
 		}
@@ -206,7 +213,7 @@ func (node *deploymentNode) setPaused(paused bool) error {
 
 		pauseNode.Spec.Paused = paused
 
-		if updateErr := sdk.Update(pauseNode); updateErr != nil {
+		if updateErr := node.client.Update(context.TODO(), pauseNode); updateErr != nil {
 			logrus.Debugf("Failed to update node resource %v: %v", pauseNode.Name, updateErr)
 			return updateErr
 		}
@@ -225,7 +232,7 @@ func (node *deploymentNode) setReplicaCount(replicas int32) error {
 	nretries := -1
 	retryErr := retry.RetryOnConflict(retry.DefaultRetry, func() error {
 		nretries++
-		if getErr := sdk.Get(&node.self); getErr != nil {
+		if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self); getErr != nil {
 			logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, getErr)
 			return getErr
 		}
@@ -236,7 +243,7 @@ func (node *deploymentNode) setReplicaCount(replicas int32) error {
 
 		node.self.Spec.Replicas = &replicas
 
-		if updateErr := sdk.Update(&node.self); updateErr != nil {
+		if updateErr := node.client.Update(context.TODO(), &node.self); updateErr != nil {
 			logrus.Debugf("Failed to update node resource %v: %v", node.self.Name, updateErr)
 			return updateErr
 		}
@@ -250,10 +257,10 @@ func (node *deploymentNode) setReplicaCount(replicas int32) error {
 }
 
 func (node *deploymentNode) replicaCount() (error, int32) {
-	nodeCopy := node.self.DeepCopy()
+	nodeCopy := &apps.Deployment{}
 
-	if err := sdk.Get(nodeCopy); err != nil {
-		logrus.Debugf("Could not get Elasticsearch node resource %v: %v", nodeCopy.Name, err)
+	if err := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, nodeCopy); err != nil {
+		logrus.Debugf("Could not get Elasticsearch node resource %v: %v", node.self.Name, err)
 		return err, -1
 	}
 
@@ -262,7 +269,7 @@ func (node *deploymentNode) replicaCount() (error, int32) {
 
 func (node *deploymentNode) waitForNodeRejoinCluster() (error, bool) {
 	err := wait.Poll(time.Second*1, time.Second*60, func() (done bool, err error) {
-		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size waiting for %v to rejoin cluster", node.name())
 			return false, getErr
@@ -276,7 +283,7 @@ func (node *deploymentNode) waitForNodeRejoinCluster() (error, bool) {
 
 func (node *deploymentNode) waitForNodeLeaveCluster() (error, bool) {
 	err := wait.Poll(time.Second*1, time.Second*60, func() (done bool, err error) {
-		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		clusterSize, getErr := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size waiting for %v to leave cluster", node.name())
 			return false, getErr
@@ -289,8 +296,8 @@ func (node *deploymentNode) waitForNodeLeaveCluster() (error, bool) {
 }
 
 func (node *deploymentNode) isMissing() bool {
-	getNode := node.self.DeepCopy()
-	if getErr := sdk.Get(getNode); getErr != nil {
+	getNode := &apps.Deployment{}
+	if getErr := node.client.Get(context.TODO(), types.NamespacedName{Name: node.name(), Namespace: node.self.Namespace}, getNode); getErr != nil {
 		if errors.IsNotFound(getErr) {
 			return true
 		}
@@ -302,12 +309,12 @@ func (node *deploymentNode) isMissing() bool {
 func (node *deploymentNode) restart(upgradeStatus *api.ElasticsearchNodeStatus) {
 
 	if upgradeStatus.UpgradeStatus.UnderUpgrade != v1.ConditionTrue {
-		if status, _ := GetClusterHealth(node.clusterName, node.self.Namespace); status != "green" {
+		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); status != "green" {
 			logrus.Infof("Waiting for cluster to be fully recovered before restarting %v: %v / green", node.name(), status)
 			return
 		}
 
-		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size prior to restart for %v", node.name())
 			return
@@ -327,12 +334,12 @@ func (node *deploymentNode) restart(upgradeStatus *api.ElasticsearchNodeStatus)
 		if replicas > 0 {
 
 			// disable shard allocation
-			if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationNone); !ok {
+			if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationNone, node.client); !ok {
 				logrus.Warnf("Unable to disable shard allocation: %v", err)
 				return
 			}
 
-			if ok, err := DoSynchronizedFlush(node.clusterName, node.self.Namespace); !ok {
+			if ok, err := DoSynchronizedFlush(node.clusterName, node.self.Namespace, node.client); !ok {
 				logrus.Warnf("Unable to perform synchronized flush: %v", err)
 			}
 
@@ -373,7 +380,7 @@ func (node *deploymentNode) restart(upgradeStatus *api.ElasticsearchNodeStatus)
 		node.refreshHashes()
 
 		// reenable shard allocation
-		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationAll); !ok {
+		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationAll, node.client); !ok {
 			logrus.Warnf("Unable to enable shard allocation: %v", err)
 			return
 		}
@@ -383,7 +390,7 @@ func (node *deploymentNode) restart(upgradeStatus *api.ElasticsearchNodeStatus)
 
 	if upgradeStatus.UpgradeStatus.UpgradePhase == api.RecoveringData {
 
-		if status, _ := GetClusterHealth(node.clusterName, node.self.Namespace); status != "green" {
+		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); status != "green" {
 			logrus.Infof("Waiting for cluster to complete recovery: %v / green", status)
 			return
 		}
@@ -397,12 +404,12 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 
 	// set our state to being under upgrade
 	if upgradeStatus.UpgradeStatus.UnderUpgrade != v1.ConditionTrue {
-		if status, _ := GetClusterHealth(node.clusterName, node.self.Namespace); status != "green" {
+		if status, _ := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); status != "green" {
 			logrus.Infof("Waiting for cluster to be fully recovered before upgrading %v: %v / green", node.name(), status)
 			return fmt.Errorf("Cluster not in green state before beginning upgrade: %v", status)
 		}
 
-		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace)
+		size, err := GetClusterNodeCount(node.clusterName, node.self.Namespace, node.client)
 		if err != nil {
 			logrus.Warnf("Unable to get cluster size prior to update for %v", node.name())
 		}
@@ -415,12 +422,12 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 		upgradeStatus.UpgradeStatus.UpgradePhase == api.ControllerUpdated {
 
 		// disable shard allocation
-		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationNone); !ok {
+		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationNone, node.client); !ok {
 			logrus.Warnf("Unable to disable shard allocation: %v", err)
 			return err
 		}
 
-		if ok, err := DoSynchronizedFlush(node.clusterName, node.self.Namespace); !ok {
+		if ok, err := DoSynchronizedFlush(node.clusterName, node.self.Namespace, node.client); !ok {
 			logrus.Warnf("Unable to perform synchronized flush: %v", err)
 		}
 
@@ -456,7 +463,7 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 		}
 
 		// reenable shard allocation
-		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationAll); !ok {
+		if ok, err := SetShardAllocation(node.clusterName, node.self.Namespace, api.ShardAllocationAll, node.client); !ok {
 			logrus.Warnf("Unable to enable shard allocation: %v", err)
 			return err
 		}
@@ -466,7 +473,7 @@ func (node *deploymentNode) update(upgradeStatus *api.ElasticsearchNodeStatus) e
 
 	if upgradeStatus.UpgradeStatus.UpgradePhase == api.RecoveringData {
 
-		if status, err := GetClusterHealth(node.clusterName, node.self.Namespace); status != "green" {
+		if status, err := GetClusterHealthStatus(node.clusterName, node.self.Namespace, node.client); status != "green" {
 			logrus.Infof("Waiting for cluster to complete recovery: %v / green", status)
 			return err
 		}
@@ -483,15 +490,15 @@ func (node *deploymentNode) executeUpdate() error {
 		// isChanged() will get the latest revision from the apiserver
 		// and return false if there is nothing to change and will update the node object if required
 		if node.isChanged() {
-			if logrus.GetLevel() == logrus.DebugLevel {
+			if logger.IsDebugEnabled() {
 				pretty, err := json.MarshalIndent(node.self, "", "  ")
 				if err != nil {
-					logrus.Debugf("Error marshalling node deployment for debug log: %v", err)
+					logger.Debugf("Error marshaling node for debug log: %v", err)
 				}
-				logrus.Debugf("Attempting to update node deployment: %+v", string(pretty))
+				logger.Debugf("Attempting to update node deployment: %+v", string(pretty))
 			}
-			if updateErr := sdk.Update(&node.self); updateErr != nil {
-				logrus.Debugf("Failed to update node resource %v: %v", node.self.Name, updateErr)
+			if updateErr := node.client.Update(context.TODO(), &node.self); updateErr != nil {
+				logger.Debugf("Failed to update node resource %v: %v", node.self.Name, updateErr)
 				return updateErr
 			}
 		}
@@ -527,12 +534,12 @@ func (node *deploymentNode) progressUnshedulableNode(upgradeStatus *api.Elastics
 }
 
 func (node *deploymentNode) refreshHashes() {
-	newConfigmapHash := getConfigmapDataHash(node.clusterName, node.self.Namespace)
+	newConfigmapHash := getConfigmapDataHash(node.clusterName, node.self.Namespace, node.client)
 	if newConfigmapHash != node.configmapHash {
 		node.configmapHash = newConfigmapHash
 	}
 
-	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace)
+	newSecretHash := getSecretDataHash(node.clusterName, node.self.Namespace, node.client)
 	if newSecretHash != node.secretHash {
 		node.secretHash = newSecretHash
 	}
@@ -543,11 +550,10 @@ func (node *deploymentNode) isChanged() bool {
 	changed := false
 
 	desired := node.self.DeepCopy()
-
 	// we want to blank this out before a get to ensure we get the correct information back (possible sdk issue with maps?)
 	node.self.Spec = apps.DeploymentSpec{}
 
-	err := sdk.Get(&node.self)
+	err := node.client.Get(context.TODO(), types.NamespacedName{Name: node.self.Name, Namespace: node.self.Namespace}, &node.self)
 	// error check that it exists, etc
 	if err != nil {
 		// if it doesn't exist, return true
@@ -561,6 +567,13 @@ func (node *deploymentNode) isChanged() bool {
 		changed = true
 	}
 
+	// check the pod's tolerations
+	if !areTolerationsSame(node.self.Spec.Template.Spec.Tolerations, desired.Spec.Template.Spec.Tolerations) {
+		logrus.Debugf("Resource '%s' has different tolerations than desired", node.self.Name)
+		node.self.Spec.Template.Spec.Tolerations = desired.Spec.Template.Spec.Tolerations
+		changed = true
+	}
+
 	// Only Image and Resources (CPU & memory) differences trigger rolling restart
 	for index := 0; index < len(node.self.Spec.Template.Spec.Containers); index++ {
 		nodeContainer := node.self.Spec.Template.Spec.Containers[index]
@@ -572,14 +585,6 @@ func (node *deploymentNode) isChanged() bool {
 			changed = true
 		}
 
-		if nodeContainer.Resources.Requests == nil {
-			nodeContainer.Resources.Requests = v1.ResourceList{}
-		}
-
-		if nodeContainer.Resources.Limits == nil {
-			nodeContainer.Resources.Limits = v1.ResourceList{}
-		}
-
 		var updatedContainer v1.Container
 		var resourceUpdated bool
 		if updatedContainer, resourceUpdated = updateResources(node, nodeContainer, desiredContainer); resourceUpdated {
